//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "debugger_private.h"
#include "mpu88xx0.h"
#include "m88100.h"
#include "m88100disasm.h"

class DebuggerMD_m88xx0 : public DebuggerMD
{
	using inherited = DebuggerMD;
 public:
	DebuggerMD_m88xx0(Debugger *parent_, m88kcpu *cpu_);
	~DebuggerMD_m88xx0() override;

	void ReqSet(uint32 flag) override { cpu->atomic_reqflag |= flag; }
	void ReqClr(uint32 flag) override { cpu->atomic_reqflag &= ~flag; }

	bool IsSuper() const override { return (cpu->psr & m88100reg::PSR_SUPER); }
	bool MMUEnabled() const override;
	uint64 TranslateAddr(saddr_t laddr) override;

	// XXX ?
	uint32 GetPC() const override { return cpu->Getxip(); }

	void SetStepOut() override;
	bool IsStepOut() const override;
	uint64 GetRegAddr(const char *name) const override;
	void BackupRegs() override;
	bool ShowRegister(Console *cons, const std::vector<std::string>& args)
		override;
	const HelpMessages& GetRegisterHelp() const override;

	BranchHistory& GetBrHist() const override { return cpu->brhist; }
	BranchHistory& GetExHist() const override { return cpu->exhist; }

	bool CheckLEA(uint32 laddr) override {
		bool rv = cpu->lastaddr == laddr;
		cpu->lastaddr = 0;	// XXX: ていうかここでクリアはおかしい
		return rv;
	}

	bool Disassemble(saddr_t laddr,
		std::string& mnemonic, std::vector<uint8>& bin) override;
	std::string FormatDisasm(const std::string& mnemonic,
		const std::vector<uint8>& bin) override;
	std::string FormatDisasmLive(const std::string& mnemonic,
		const std::vector<uint8>& bin) override;

 private:
	m88kcpu *cpu = NULL;		// CPU コア
	m88100reg prev {};

	// 前回アクセス時の論理ページ。.addr == 0xffffffff なら無効。
	// [0] が Inst、[1] が Data XXX とりあえず
	saddr_t last_lpage[2];
	uint32  last_ppage[2];

	static bool IsBBTB0(uint32 op);
	static bool IsBB1(uint32 op);
	static bool IsTB1(uint32 op);
	static bool IsBBTB1(uint32 op);
	const char *CondStr(uint32 op);

	static const HelpMessages HelpMsgRegister;
	bool MatchCMMUCmd(const std::string& cmdname, const char *name, int *);
	void ShowRegMain(Console *cons);
	void ShowRegCache(Console *cons, const std::vector<std::string>& args,
		int id);
	void ShowRegCacheOverview(Console *cons, int id);
	void ShowRegCacheSet(Console *cons, int id, int setidx);
	void ShowRegCtrl(Console *cons);
	void ShowRegFPU(Console *cons);
	void ShowRegMMU(Console *cons, int id);
	void ShowRegOther(Console *cons);

	// cmp 命令後の分岐命令で結果レジスタを表示するため、
	// cmp 命令のデスティネーションレジスタを覚えておく。
	// -1 なら該当しないの意。
	int cmp_rd;

	uint32 so_r1 = 0;		// ステップアウト判定用
};
