//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

// 逆アセンブラ(共通部分)

#pragma once

#include "debugger_private.h"

// 逆アセンブラ抽象クラス
class Disasm
{
 public:
	// コンストラクタ
	Disasm(DebuggerMD *m) {
		md = m;
	}
	virtual ~Disasm() { }

	// laddr の位置を1命令逆アセンブルする。
	// 成功すれば true、失敗すれば false を返す?
	// 成功すれば text に出力文字列、ir に命令バイナリがセットされている。
	//
	// Exec() 実装は、laddr が論理アドレスなら md->TranslateAddr() を呼んで
	// 物理アドレスを取得し、vm_phys_peek8() で物理メモリを読み出すこと。
	virtual bool Exec(saddr_t laddr) = 0;

	// 出力文字列(ダンプ)
	std::string dump {};

	// 出力文字列(命令)
	std::string text {};

	// この命令のバイナリ列
	// (メモリイメージ順)
	std::vector<uint8> bin {};

 protected:
	// デバッガ(MD)クラス。TranslateAddr() のため。
	DebuggerMD *md {};

	// 継承先の Exec() の先頭で呼び出してください。
	void ExecInit() {
		text.clear();
		bin.clear();
	}
};
