//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "netdriver.h"
#include "mainapp.h"
#include "mystring.h"
#include "mythread.h"

static void *netdriver_run(void *);

std::unique_ptr<NetDriver> gNetDriver;

// コンストラクタ
NetDriver::NetDriver(const char *drivername_, EthernetDevice *parent_,
	std::mutex *mtx_, std::condition_variable *cv_)
{
	logname = "netdriver";
	devname = "NetDriver";

	monitor_size = nnSize(30, 7);

	fd = -1;
	drivername = drivername_;
	parent = parent_;
	mtx = mtx_;
	cv = cv_;
}

// デストラクタ
NetDriver::~NetDriver()
{
}

// モニタ
void
NetDriver::MonitorUpdate(TextScreen& monitor)
{
	monitor.Clear();

	monitor.Print(0, 0, "NetDriver: %s", drivername);
	monitor.Print(0, 1, "Device   : %s", devpath.c_str());
	monitor.Print(0, 2, "Interface: %s", ifname.c_str());

	monitor.Print(5, 4, "%9s %9s", "Packets", "Bytes");
	monitor.Print(0, 5, "Send %9" PRIu64 " %9" PRIu64, tx_pkts, tx_bytes);
	monitor.Print(0, 6, "Recv %9" PRIu64 " %9" PRIu64, rx_pkts, rx_bytes);
}

// スレッドが必要な継承クラスが呼ぶ
bool
NetDriver::InitRecvThread()
{
	// 受信スレッド起動
	pthread_t th;
	pthread_create(&th, NULL, netdriver_run, NULL);

	return true;
}

// 受信スレッドのエントリポイント
void *
netdriver_run(void *dummy)
{
	PTHREAD_SETNAME("netdriver");
	pthread_detach(pthread_self());

	gNetDriver->ThreadRun();
	return NULL;
}

// ifup スクリプト実行
bool
NetDriver::Run_ifup()
{
	return RunScript("nono-ifup");
}

// ifdown スクリプト実行
bool
NetDriver::Run_ifdown()
{
	return RunScript("nono-ifdown");
}

// スクリプトを実行
bool
NetDriver::RunScript(const char *filename)
{
	// ファイルが存在するか
	std::string path = gMainApp.SearchFile(filename);
	if (path.empty()) {
		warnx("%s not found", filename);
		return false;
	}

	// コマンドラインを作成
	std::string cmd = path + " " + ifname;

	// 実行
	int rv = system(cmd.c_str());
	if (rv == -1) {
		warn("%s: system() failed", path.c_str());
		return false;
	}
	if (rv == 127) {
		warnx("%s: system(): shell failed", path.c_str());
		return false;
	}
	if (rv != 0) {
		warnx("%s failed with %d", path.c_str(), rv);
		return false;
	}
	return true;
}
