//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "logger.h"
#include "mpu.h"

// グローバルインスタンス
Logger gLogger;

// コンストラクタ
Logger::Logger()
{
}

// デストラクタ
Logger::~Logger()
{
}

// ログ書き込み
void
Logger::Write(const char *str)
{
	std::unique_lock<std::mutex> lk(mtx);
	// パイプへ出力 (EOL まで出力する)
	for (const char *s = str; ; s++) {
		// 一杯なら古いのを捨てて書き込み続ける
		if (pipe.Enqueue(*s) == false) {
			uint8 dummy;
			pipe.Dequeue(&dummy);
			pipe.Enqueue(*s);
		}
		if (*s == '\0')
			break;
	}
	// 必要なら標準出力にも出力
	if (use_stdout) {
		printf("%s\n", str);
	}
}

// ログ読み出し
bool
Logger::Read(char *buf, int bufsize)
{
	uint8 ch;
	int i;

	std::unique_lock<std::mutex> lk(mtx);
	for (i = 0; i < bufsize - 1 && pipe.Dequeue(&ch); ) {
		buf[i++] = ch;
		if (ch == '\0') {
			break;
		}
	}
	// buf が足りない場合ここで打ち切り
	if (i == bufsize - 1) {
		// ただし EOL 含めて bufsize ぴったりなら取り出しておく
		if (pipe.Peek(0) == '\0') {
			pipe.Dequeue(&ch);
		}
		buf[i] = '\0';
	}
	return (i != 0);
}


// パニック
void
panic(const char *funcname, const char *fmt, ...)
{
	char fmtbuf[1024];
	va_list ap;

	va_start(ap, fmt);
	vsnprintf(fmtbuf, sizeof(fmtbuf), fmt, ap);
	va_end(ap);

	// panic の場合ここでプロセス終了するのでログではなく stderr に出力
	fprintf(stderr, "panic! PC=$%08x %s at %s\n",
		gMPU->GetPPC(), fmtbuf, funcname);
	abort();
}
