//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "mystring.h"
#include <algorithm>

std::string
string_format(const char *fmt, ...)
{
	va_list ap;
	char *buf;

	va_start(ap, fmt);
	vasprintf(&buf, fmt, ap);
	va_end(ap);
	std::string rv(buf);
	free(buf);

	return rv;
}

// 文字列 str から先頭の連続する空白文字を取り除いた新しい文字列を返す。
std::string
string_ltrim(const std::string& str)
{
	auto it = str.begin();
	for (; it != str.end(); it++) {
		if (!isspace(*it))
			break;
	}
	return std::string(it, str.end());
}

// 文字列 str から末尾の連続する空白文字を取り除く (str を書き換える)。
void
string_rtrim(std::string& str)
{
	while (isspace(*str.rbegin())) {
		str.pop_back();
	}
}

// 文字列 str から末尾の連続する空白文字を取り除く (str を書き換える)。
void
rtrim(char *str)
{
	char *p = strchr(str, '\0');
	while (--p >= str && (isspace((int)*p) || *p == '\r' || *p == '\n')) {
		*p = '\0';
	}
}

// 文字列 src 中の ASCII 大文字を小文字にした新しい文字列を返す。
std::string
string_tolower(const std::string& src)
{
	std::string dst(src);
	std::transform(dst.begin(), dst.end(), dst.begin(),
		[](unsigned char c){ return std::tolower(c); });
	return dst;
}

#if 0
#include <cstdio>
#include <sys/time.h>
int main()
{
	std::string s0 = string_format("%d_%d", 0, 1);
	if (s0 != "0_1") {
		printf("error\n");
		return 1;
	}
	timeval start, end, result;
	gettimeofday(&start, NULL);
	for (int i = 0; i < 100000; i++) {
		std::string s = string_format("%d_%d", i, i);
	}
	gettimeofday(&end, NULL);
	timersub(&end, &start, &result);
	long long t = (result.tv_sec) * 1000000 + result.tv_usec;
	printf("%d.%03d\n", (int)t / 1000, (int)t % 1000);
	return 0;
}
#endif
