//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "object.h"
#include "logger.h"
#include "mpu.h"
#include "scheduler.h"
#include "textscreen.h"

// 全オブジェクトのリスト
std::vector<Object*> gObjects;

// コンストラクタ
Object::Object()
{
	logname = "?";
	devname = "?";

	// 自分自身をリストに繋ぐ
	gObjects.push_back(this);
}

// デストラクタ
Object::~Object()
{
}

// メッセージ表示
// 常に表示 (あらかじめ loglevel の評価をした後で呼び出す用)
void
Object::putmsg(const char *fmt, ...) const
{
	va_list ap;

	va_start(ap, fmt);
	vputmsg(fmt, ap);
	va_end(ap);
}

// メッセージ出力 (va_list)
// 常に表示 (あらかじめ loglevel の評価をした後で呼び出す用)
void
Object::vputmsg(const char *fmt, va_list ap) const
{
	char buf[1024];
	int len;

	// デバイス名だけ生表示されても分かりづらいので括弧でもつけておく
	len = snprintf(buf, sizeof(buf), "(%s) ", devname.c_str());
	vsnprintf(buf + len, sizeof(buf) - len, fmt, ap);
	gLogger.Write(buf);
}

// ログ出力 (可変引数)
// 常に表示 (あらかじめ loglevel の評価をした後で呼び出す用)
void
Object::putlog(const char *fmt, ...) const
{
	va_list ap;

	va_start(ap, fmt);
	vputlog(fmt, ap);
	va_end(ap);
}

// ログ出力 (va_list)
// 常に表示 (あらかじめ loglevel の評価をした後で呼び出す用)
void
Object::vputlog(const char *fmt, va_list ap) const
{
	char buf[1024];
	uint64 vt;
	int len = 0;

	vt = gScheduler->GetVirtTime();
	len += snprintf(buf + len, sizeof(buf) - len, "%4d.%03d'%03d'%03d ",
		(int)(vt / 1000 / 1000 / 1000),
		(int)((vt / 1000 / 1000) % 1000),
		(int)((vt / 1000) % 1000),
		(int)(vt % 1000));

	len += snprintf(buf + len, sizeof(buf) - len, "%08x %s ",
		gMPU->GetPPC(), devname.c_str());
	vsnprintf(buf + len, sizeof(buf) - len, fmt, ap);

	gLogger.Write(buf);
}
