//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "textscreen.h"
#include <vector>

//
// 基本オブジェクト
//
// ログ出力機能を持つ。
//
class Object : public IMonitor
{
 public:
	Object();
	virtual ~Object() override;

	//
	// ログ
	//

	// ログ識別名
	std::string logname;
	// ログレベル
	// 0: 未実装動作などのデバッグログではない情報
	// 1: 大雑把なデバッグログ
	// 2: 詳細なデバッグログ
	// 3: うるさいデバッグログ
	int loglevel = 0;

	// メッセージ出力
	// loglevel が lv 以上の時だけ表示。
	// こちらは仮想時刻、現在の PC を表示しない。Init() など VM 実行前や
	// 仮想マシン(特に仮想時刻、PC) に関係ないあたりはこちらを使用。
	inline void
	putmsg(int lv, const char *fmt, ...) __printflike(3, 4)
	{
		if (__predict_false(loglevel >= lv)) {
			va_list ap;

			va_start(ap, fmt);
			vputmsg(fmt, ap);
			va_end(ap);
		}
	}
	// メッセージ出力
	void putmsg(const char *fmt, ...) const __printflike(2, 3);
	void vputmsg(const char *fmt, va_list ap) const __printflike(2, 0);

	// ログ出力
	// loglevel が lv 以上の時だけ表示。
	// 仮想時刻と現在の PC も表示する。VM 実行中は基本的にこれで表示。
	inline void
	putlog(int lv, const char *fmt, ...) const __printflike(3, 4)
	{
		if (__predict_false(loglevel >= lv)) {
			va_list ap;

			va_start(ap, fmt);
			vputlog(fmt, ap);
			va_end(ap);
		}
	}
	// ログ出力(ログレベル限定なし)
	void putlog(const char *fmt, ...) const __printflike(2, 3);
	void vputlog(const char *fmt, va_list ap) const __printflike(2, 0);

	// デバイス名(デバッグ表示用)
	// XXX オブジェクトなのにデバイス名とは一体...
	std::string devname;

	//
	// モニター
	//

	// モニター更新 (派生クラス側で用意)
	void MonitorUpdate(TextScreen&) override { }
	nnSize GetMonitorSize() override { return monitor_size; }
	nnSize monitor_size {};
};

// 全オブジェクトのリスト
extern std::vector<Object*> gObjects;
