//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "bus.h"
#include "ram.h"

IODevice *devtable[256];

//
// 物理バスアクセス?
// addr はそれぞれアラインされていること
//

uint64
vm_phys_read_8(uint32 addr)
{
	uint64 data;

	if (addr < ram_size) {
		data = ram[HLB(addr)];
	} else {
		IODevice *dev = ::devtable[addr >> 24];
		data = dev->Read8(addr);
	}
	return data;
}

uint64
vm_phys_read_16(uint32 addr)
{
	uint64 data;

	if (__predict_false(addr & 1))
		PANIC("unaligned access $%08x", addr);

	if (addr < ram_size) {
		data = *(uint16 *)&ram[HLW(addr)];
	} else {
		IODevice *dev = ::devtable[addr >> 24];
		data = dev->Read16(addr);
	}
	return data;
}

uint64
vm_phys_read_32(uint32 addr)
{
	uint64 data;

	if (__predict_false(addr & 3))
		PANIC("unaligned access $%08x", addr);

	if (addr < ram_size) {
		data = *(uint32 *)&ram[addr];
	} else {
		IODevice *dev = ::devtable[addr >> 24];
		data = dev->Read32(addr);
	}
	return data;
}

uint64
vm_phys_write_8(uint32 addr, uint32 data)
{
	if (addr < ram_size) {
		ram[HLB(addr)] = data;
		return 0;
	} else {
		IODevice *dev = ::devtable[addr >> 24];
		return dev->Write8(addr, data);
	}
}

uint64
vm_phys_write_16(uint32 addr, uint32 data)
{
	if (__predict_false(addr & 1))
		PANIC("unaligned access $%08x", addr);

	if (addr < ram_size) {
		*(uint16 *)&ram[HLW(addr)] = data;
		return 0;
	} else {
		IODevice *dev = ::devtable[addr >> 24];
		return dev->Write16(addr, data);
	}
}

uint64
vm_phys_write_32(uint32 addr, uint32 data)
{
	if (__predict_false(addr & 3))
		PANIC("unaligned access $%08x", addr);

	if (addr < ram_size) {
		*(uint32 *)&ram[addr] = data;
		return 0;
	} else {
		IODevice *dev = ::devtable[addr >> 24];
		return dev->Write32(addr, data);
	}
}

uint64
vm_phys_peek_8(uint32 addr)
{
	uint64 data;

	if (addr < ram_size) {
		data = ram[HLB(addr)];
	} else {
		IODevice *dev = ::devtable[addr >> 24];
		data = dev->Peek8(addr);
	}
	return data;
}
