//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"
#include "scheduler.h"

struct CRTC
{
	uint16 r[24];
	uint16 op;

	static const uint32 R00		= 0x00;		// $E80000
	static const uint32 R01		= 0x01;		// $E80002
	static const uint32 R02		= 0x02;		// $E80004
	static const uint32 R03		= 0x03;		// $E80006
	static const uint32 R04		= 0x04;		// $E80008
	static const uint32 R05		= 0x05;		// $E8000A
	static const uint32 R06		= 0x06;		// $E8000C
	static const uint32 R07		= 0x07;		// $E8000E
	static const uint32 R08		= 0x08;		// $E80010
	static const uint32 R09		= 0x09;		// $E80012
	static const uint32 R10		= 0x0a;		// $E80014
	static const uint32 R11		= 0x0b;		// $E80016
	static const uint32 R12		= 0x0c;		// $E80018
	static const uint32 R13		= 0x0d;		// $E8001A
	static const uint32 R14		= 0x0e;		// $E8001C
	static const uint32 R15		= 0x0f;		// $E8001E
	static const uint32 R16		= 0x10;		// $E80020
	static const uint32 R17		= 0x11;		// $E80022
	static const uint32 R18		= 0x12;		// $E80024
	static const uint32 R19		= 0x13;		// $E80026
	static const uint32 R20		= 0x14;		// $E80028
	static const uint32 R21		= 0x15;		// $E8002A
	static const uint32 R22		= 0x16;		// $E8002C
	static const uint32 R23		= 0x17;		// $E8002E
	static const uint32 REG_END	= 0x18;
	static const uint32 REG_OP 	= 0x240;	// $E80480
};

class CRTCDevice
	: public IODevice
{
	using inherited = IODevice;
 public:
	CRTCDevice();
	~CRTCDevice() override;

	void ResetHard() override;

 protected:
	// BusIO インタフェース
	static const uint NPORT = 0x800;
	uint64 Read(uint32 addr);
	uint64 Write(uint32 addr, uint32 data);
	uint64 Peek(uint32 addr);

 private:
	void SetReg(uint32 offset, uint32 data);

	// 垂直表示期間イベント
	void VDispCallback(int arg);

	struct CRTC crtc {};

	Event vdisp_event {};
};

extern std::unique_ptr<CRTCDevice> gCRTC;
