//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "keyboard.h"
#include "mystring.h"

std::unique_ptr<Keyboard> gKeyboard;

// コンストラクタ
Keyboard::Keyboard()
{
	logname = "keyboard";
	devname = "Keyboard";
}

// デストラクタ
Keyboard::~Keyboard()
{
}

// キー名を返す (主にデバッグ用)
const std::string
Keyboard::GetName(uint code)
{
	if (code >= keyname.size()) {
		return string_format("(keycode=0x%x, out of range)", code);
	}
	if (keyname[code] == NULL) {
		return string_format("(keycode=0x%x)", code);
	}
	return std::string(keyname[code]);
}

/*static*/ std::vector<const char *>
Keyboard::keyname {
	"(none)",		// 00
	"BREAK",		// 01
	"COPY",			// 02
	"romaji",		// 03
	"code",			// 04
	"kigou",		// 05
	"touroku",		// 06
	"HELP",			// 07
	"HOME",			// 08
	"TAB",			// 09
	"CTRL",			// 0a
	"kana",			// 0b
	"SHIFT_L",		// 0c
	"SHIFT_R",		// 0d
	"CAPS",			// 0e
	"SF",			// 0f

	"ESC",			// 10
	"BS",			// 11
	"enter",		// 12
	"INS",			// 13
	"space",		// 14
	"DEL",			// 15
	"XFER",			// 16
	"VALID",		// 17
	"PF11",			// 18
	"PF12",			// 19
	"PF13",			// 1a
	"PF14",			// 1b
	"up",			// 1c
	"left",			// 1d
	"right",		// 1e
	"down",			// 1f

	"ROLLUP",		// 20
	"ROLLDOWN",		// 21
	"1",			// 22
	"2",			// 23
	"3",			// 24
	"4",			// 25
	"5",			// 26
	"6",			// 27
	"7",			// 28
	"8",			// 29
	"9",			// 2a
	"0",			// 2b
	"minus",		// 2c
	"circum",		// 2d
	"backslash",	// 2e
	"UNDO",			// 2f

	"OPT1",			// 30
	"OPT2",			// 31
	"Q",			// 32
	"W",			// 33
	"E",			// 34
	"R",			// 35
	"T",			// 36
	"Y",			// 37
	"U",			// 38
	"I",			// 39
	"O",			// 3a
	"P",			// 3b
	"at",			// 3c
	"bracketleft",	// 3d
	"hira",			// 3e
	"zenkaku",		// 3f

	NULL,			// 40
	NULL,			// 41
	"A",			// 42
	"S",			// 43
	"D",			// 44
	"F",			// 45
	"G",			// 46
	"H",			// 47
	"J",			// 48
	"K",			// 49
	"L",			// 4a
	"semicolon",	// 4b
	"colon",		// 4c
	"bracketright",	// 4d
	NULL,			// 4e
	NULL,			// 4f

	NULL,			// 50
	NULL,			// 51
	"Z",			// 52
	"X",			// 53
	"C",			// 54
	"V",			// 55
	"B",			// 56
	"N",			// 57
	"M",			// 58
	"comma",		// 59
	"period",		// 5a
	"slash",		// 5b
	"underscore",	// 5c
	NULL,			// 5d
	NULL,			// 5e
	NULL,			// 5f

	NULL,			// 60
	"PAD_plus",		// 61
	"PAD_minus",	// 62
	"PAD_7",		// 63
	"PAD_8",		// 64
	"PAD_9",		// 65
	"PAD_4",		// 66
	"PAD_5",		// 67
	"PAD_6",		// 68
	"PAD_1",		// 69
	"PAD_2",		// 6a
	"PAD_3",		// 6b
	"PAD_0",		// 6c
	"PAD_decimal",	// 6d
	"PAD_enter",	// 6e
	"PAD_CLR",		// 6f

	NULL,			// 70
	NULL,			// 71
	"F1",			// 72
	"F2",			// 73
	"F3",			// 74
	"F4",			// 75
	"F5",			// 76
	"F6",			// 77
	"F7",			// 78
	"F8",			// 79
	"F9",			// 7a
	"F10",			// 7b
	"PAD_multiply",	// 7c
	"PAD_divide",	// 7d
	"PAD_equal",	// 7e
	"PAD_comma",	// 7f

	"XF1",			// 80
	"XF2",			// 81
	"XF3",			// 82
	"XF4",			// 83
	"XF5",			// 84
};
