//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

// MPU 共通部

#pragma once

#include "device.h"

class MPUDevice : public Device
{
	using inherited = Device;
 public:
	MPUDevice();
	~MPUDevice() override;

	// MPU を request サイクル分実行する
	virtual uint32 Run(uint64 request) = 0;

	// 基準サイクル数を取得
	virtual uint64 total_cycle() const = 0;

	// MPU の処理をこの命令で中断させる。
	// スケジューラのイベントリストに追加が発生した時に VM 側から
	// MPU へ処理の中断を指示する。
	virtual void Release() = 0;

	// 現在実行中の命令の PC を取得
	virtual uint32 GetPPC() const = 0;

	// 現在アクセス中の論理アドレス、物理アドレスを取得。
	// Laddr == Paddr が MMU オフなのか PA=VA で運用中なのかの区別は
	// ここでは付かない。
	virtual uint32 GetLaddr() const = 0;
	virtual uint32 GetPaddr() const = 0;

	// 割り込み発生を MPU に通知 (オートベクタ)
	virtual void Interrupt(Object *src, int level) = 0;
};

extern std::unique_ptr<MPUDevice> gMPU;
