//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

// MPU (m680x0)

#pragma once

#include "mpu.h"
#include "m68030.h"

class MPU680x0Device : public MPUDevice
{
	using inherited = MPUDevice;
 public:
	MPU680x0Device(uint32);
	~MPU680x0Device() override;

	bool Init() override;
	void ResetHard() override;
	void MonitorUpdate(TextScreen&) override;

	uint32 Run(uint64 request) override;

	// 現在の PPC を取得
	uint32 GetPPC() const override { return RegPPC; }

	// 現在アクセス中の論理アドレス、物理アドレスを取得。
	uint32 GetLaddr() const override { return cpu->bus.laddr; }
	uint32 GetPaddr() const override { return cpu->bus.paddr; }

	// 割り込み発生を MPU に通知 (オートベクタ)
	void Interrupt(Object *src, int level) override;

	// 割り込み発生を MPU に通知 (ベクタ)
	void Interrupt(Object *src, int level, int vector);

	// MPU の処理をこの命令で中断させる。
	// スケジューラのイベントリストに追加が発生した時に VM 側から
	// MPU へ処理の中断を指示する。
	void Release() override { cpu->atomic_reqflag |= CPU_REQ_RELEASE; }

	// 基準サイクル数を取得
	uint64 total_cycle() const override { return cpu->total_cycle; }

	// A-Line, F-Line 命令エミュレーションのコールバックを指定する。
	// (Human68k エミュレーションで使用)
	void SetALineCallback(bool (*callback)(m68kcpu*, void*), void *arg);
	void SetFLineCallback(bool (*callback)(m68kcpu*, void*), void *arg);

	// ダブルバスフォールトのコールバックを指定する。
	void SetHaltCallback(void (*callback)(void*), void *arg);

	// FPU を持っているか
	bool HaveFPU() const {
		return cpu->has_fpu;
	}

	// CPU コアを取得 (主にデバッグ用途)
	m68kcpu *GetCPU() const {
		return cpu;
	}

 private:
	m68kcpu *cpu = NULL;
};

// ATC モニタ
class MPUATC : public Object
{
 public:
	MPUATC(m68kcpu *cpu);
	~MPUATC() override { }

	void MonitorUpdate(TextScreen&) override;

 private:
	m68kcpu *cpu = NULL;
};

extern std::unique_ptr<MPUATC> gMPUATC;

#define gMPU680x0	(dynamic_cast<MPU680x0Device *>(gMPU.get()))
