//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

// MPU (m88xx0)

#pragma once

#include "mpu.h"
#include "m88100.h"

class MPU88xx0Device : public MPUDevice
{
	using inherited = MPUDevice;
 public:
	MPU88xx0Device(uint32 reset_addr);
	~MPU88xx0Device() override;

	void ResetHard() override;

	void MonitorUpdate(TextScreen&) override;

	// MPU を request サイクル分実行する
	uint32 Run(uint64 request) override { return cpu->Run(request); }

	// 基準サイクル数を取得
	uint64 total_cycle() const override { return cpu->total_cycle; }

	// MPU の処理をこの命令で中断させる。
	void Release() override { cpu->Release(); }

	// 現在実行中の命令の PC を取得
	uint32 GetPPC() const override { return cpu->Getxip(); }

	// 現在アクセス中の論理アドレス、物理アドレスを取得。
	uint32 GetLaddr() const override { return 0; }	// XXX NotImplemented
	uint32 GetPaddr() const override { return 0; }	// XXX NotImplemented

	// 割り込み発生を MPU に通知
	void Interrupt(Object *src, int level) override;

	// CPU コアを取得 (主にデバッグ用途)
	m88kcpu *GetCPU() const { return cpu.get(); }

 private:
	// リセットベクタ
	uint32 reset_vector;

	std::unique_ptr<m88kcpu> cpu;
};

// ATC モニタ
class MPU88200ATC : public Object
{
 public:
	MPU88200ATC(m88200 *cmmu);
	~MPU88200ATC() override { }

	void MonitorUpdate(TextScreen&) override;

 private:
	m88200 *cmmu = NULL;
};

extern std::unique_ptr<MPU88200ATC> gMPU88200ATC[2];

#define gMPU88xx0	(dynamic_cast<MPU88xx0Device *>(gMPU.get()))
