//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "rom.h"
#include "mainapp.h"
#include "mystring.h"

//
// ROM 共通部
//
// ROM はビッグエンディアンのままメモリに置かれるので、
// 必要ならアクセス時に都度変換すること。
//

ROMDevice::ROMDevice()
{
	logname = "rom";

	mask = 0xffffffff;
}

ROMDevice::~ROMDevice()
{
}

// name の ROM を探してロードする。
// name はパスを含まないファイル名のみ。パスはこちらで探す。
// mem は file をさす。
// 失敗した場合はエラーメッセージを表示して false を返す。
bool
ROMDevice::LoadROM(const char *name, int size)
{
	filename = gMainApp.SearchFile(name);
	if (filename.empty()) {
		warnx("%s not found", name);
		return false;
	}

	// オープン
	file.SetFilename(filename);
	file.SetDispname(string_format("ROM \"%s\"", filename.c_str()));
	mem = file.OpenRdOnly((off_t)size);
	if (mem == NULL) {
		return false;
	}

	return true;
}

uint64
ROMDevice::Read8(uint32 addr)
{
	return mem[addr & mask];
}

uint64
ROMDevice::Read16(uint32 addr)
{
	uint16 data;
	data = *(uint16 *)&mem[addr & mask];
	return be16toh(data);
}

uint64
ROMDevice::Read32(uint32 addr)
{
	uint32 data;
	data = *(uint32 *)&mem[addr & mask];
	return be32toh(data);
}

uint64
ROMDevice::Write8(uint32 addr, uint32 data)
{
	return (uint64)-1;
}

uint64
ROMDevice::Write16(uint32 addr, uint32 data)
{
	return (uint64)-1;
}

uint64
ROMDevice::Write32(uint32 addr, uint32 data)
{
	return (uint64)-1;
}

uint64
ROMDevice::Peek8(uint32 addr)
{
	return mem[addr & mask];
}
