//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"

class Luna88kPROMEmuDevice : public IODevice
{
	using inherited = IODevice;

	// このデバイス先頭アドレス
	static const uint32 baseaddr		= 0x41000000;

	// 謎の I/O 空間
	static const uint32 ROMIO_BASE		= 0x41010000;
	static const uint32 ROMIO_IPL		= (ROMIO_BASE + 0);

 public:
	Luna88kPROMEmuDevice();
	~Luna88kPROMEmuDevice() override;

	bool Init() override;
	void ResetHard() override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

 private:
	uint64 Decoder(uint32 addr) const;

	// 起動時の処理を行う
	uint32 IPL();

	// 自前 ROM 部分
	std::unique_ptr<uint8[]> romdata {};
	uint32 romdatalen {};

	// リセット後 IPL を完了したら true
	bool ipl_done = false;
};
