//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "sysport.h"
#include "mfp.h"
#include "sram.h"

SysportDevice::SysportDevice()
{
	logname = "sysport";
	devname = "Sysport";
	devaddr = baseaddr;
	devlen  = 0x2000;
}

SysportDevice::~SysportDevice()
{
}

uint64
SysportDevice::Read(uint32 addr)
{
	uint32 data;

	switch (addr) {
	 case SYSPORT::CONTRAST:
		data = 0xf0 | sysport.contrast;
		break;
	 case SYSPORT::SCOPE3D:
		data = 0xff;
		break;
	 case SYSPORT::IMAGEUNIT:
		data = 0xff;
		break;
	 case SYSPORT::KEY:
		// キーボードは接続されてることにする
		data = 0x08;
		break;
	 case SYSPORT::WAIT:
		data = 0xff;
		break;
	 case SYSPORT::MPU:
		data = 0xdc;
		break;
	 case SYSPORT::SRAMWP:
		data = 0xff;
		break;
	 case SYSPORT::POWEROFF:
		data = 0xff;
		break;
	 default:
		__unreachable();
	}
	putlog(1, "$%06X -> $%02x", addr, data);
	return data;
}

uint64
SysportDevice::Write(uint32 addr, uint32 data)
{
	switch (addr) {
	 case SYSPORT::CONTRAST:
		sysport.contrast = data & 15;
		break;

	 case SYSPORT::SCOPE3D:
	 case SYSPORT::IMAGEUNIT:
		break;

	 case SYSPORT::KEY:
		data &= 0x0e;
		// bit1,2
		if ((data & 0x06)) {
			putlog(0, "$E8E007 未サポート書き込み $%02x", data);
		}
		// bit3 KEY CTRL は MFP で管理する
		gMFP->SetKeyCtrl(data & 0x08);
		break;

	 case SYSPORT::WAIT:
		// 設定値で保持
		sysport.rom_wait = data >> 4;
		sysport.ram_wait = data;
		break;

	 case SYSPORT::MPU:
		break;

	 case SYSPORT::SRAMWP:
		if (data == 0x31) {
			gSRAM->write_enable(true);
		} else {
			gSRAM->write_enable(false);
		}
		break;

	 case SYSPORT::POWEROFF:
		putlog(0, "$E8E00F (POWEROFF) 未サポート書き込み %02x", data);
		break;

	 default:
		__unreachable();
	}
	return 0;
}

uint64
SysportDevice::Peek(uint32 addr)
{
	uint32 data;

	switch (addr) {
	 case SYSPORT::CONTRAST:
		data = 0xf0 | sysport.contrast;
		break;
	 case SYSPORT::SCOPE3D:
		data = 0xff;
		break;
	 case SYSPORT::IMAGEUNIT:
		data = 0xff;
		break;
	 case SYSPORT::KEY:
		data = 0xff;
		break;
	 case SYSPORT::WAIT:
		data = 0xff;
		break;
	 case SYSPORT::MPU:
		data = 0xdc;
		break;
	 case SYSPORT::SRAMWP:
		data = 0xff;
		break;
	 case SYSPORT::POWEROFF:
		data = 0xff;
		break;
	 default:
		__unreachable();
	}
	return data;
}
