//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

// TAS ポート

#include "tas.h"

std::unique_ptr<TASDevice> gTAS;

// コンストラクタ
TASDevice::TASDevice()
{
	logname = "tas";
	devname = "TAS";
}

// デストラクタ
TASDevice::~TASDevice()
{
}

// XXX: RMW サイクルが発行される前提のレジスタなので、
// CPU / バス 側に RMW を実装しなければならない。

uint64
TASDevice::Read(uint32 addr)
{
	putlog(2, "TAS 読み込み 0x%02x", (uint8)tas);
	return tas;
}

uint64
TASDevice::Write(uint32 addr, uint32 data)
{
	// bit7 のみ有効
	// LUNA-I では B 接続なので、各バイトの bit7 に相当。
	// LUNA88K では BFFF 接続なので bit31 に相当。
	// 最上位ビット以外のビットは「不定」。
	tas = data;
	putlog(1, "TAS 書き込み 0x%02x", data);
	return 0;
}

#if 0
// RMW はこんな感じ?
uint64
TASDevice::ReadWrite(uint32 addr, uint32 data)
{
	uint8 rv;
	rv = tas.exchange(data);
	putlog(1, "TAS RMW 0x%02 to 0x%02", rv, data);
	return rv;
}
#endif

uint64
TASDevice::Peek(uint32 addr)
{
	return tas;
}
