//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "vm_x68k.h"
#include "bus.h"
#include "config.h"
#include "human68k.h"
#include "mpu680x0.h"
#include "renderer.h"
#include "rp5c15.h"
#include "scheduler.h"
#include "x68kkbd.h"
#include "xiospace.h"

//
// X68030 VM
//

// コンストラクタ
VM_X68030::VM_X68030()
{
	// 機種固有パラメータ
	// MPU クロックの初期値は機種ごとに異なる。単位は MHz
	gConfig->SetDefault("mpu-clock", 25);
	// RAM 初期値は MAX にしておく
	gConfig->SetDefault("ram-size", 12);
	// SPC の入力クロック (5MHz = 200ns)
	gConfig->Add("!spc-tCLF", 200);

	// デバイス (順序に注意)
	// BusErrDevice は親コンストラクタで作成済み。

	gScheduler.reset(new Scheduler());				// required by *
	gMPU.reset(new MPU680x0Device(0xff0000));
	gRAM.reset(new RAMDevice());					// required by XIOSpace
	xiospace.reset(new XIOSpaceDevice());

	gKeyboard.reset(new X68030Keyboard());
	gRenderer.reset(new X68030Renderer());

	// 割り当て
	// 今は拡張メモリがないので、今は上位8ビット全領域がミラーになっている。
	for (int i = 0; i < countof(::devtable); i++) {
		::devtable[i] = xiospace.get();
	}
}

// デストラクタ
VM_X68030::~VM_X68030()
{
}

// X68030 の電源ボタンを押す。
void
VM_X68030::PowerButton()
{
	// 電源	ボタン
	// ON	OFF->ON		電源オフ割り込みをネゲート?
	// ON	ON ->OFF	電源オフ割り込みをアサート?
	// OFF	OFF->ON		電源オン
	// OFF	ON ->OFF	(通常はあり得ないはず)

	// ボタンの状態を更新して..
	power_button = !power_button;

	// デバイスに指示
	DevicePowerOn();
}


//
// RXZ .r .x .z human68k console emulator
//

// コンストラクタ
VM_RXZ::VM_RXZ()
{
	// 機種固有パラメータ
	// MPU クロックの初期値は機種ごとに異なる。単位は MHz
	gConfig->SetDefault("mpu-clock", 25);
	// RAM 初期値は MAX にしておく
	gConfig->SetDefault("ram-size", 12);
	// SPC の入力クロック (5MHz = 200ns)
	gConfig->Add("!spc-tCLF", 200);

	// デバイス (順序に注意)
	// BusErrDevice は親コンストラクタで作成済み。

	gScheduler.reset(new Scheduler());				// required by *
	gMPU.reset(new MPU680x0Device(0));
	gRAM.reset(new RAMDevice());					// required by Human68k
	gRTC.reset(new RP5C15Device());
	human68k.reset(new Human68k());
}

// デストラクタ
VM_RXZ::~VM_RXZ()
{
}

void
VM_RXZ::PowerButton()
{
	// RXZ の電源ボタンは電源オンにしか使わない。
	//
	// 電源	ボタン
	// ON	PUSH		(何も起きない)
	// OFF	PUSH		電源オン

	// デバイスに指示
	DevicePowerOn();
}
