//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"
#include <mutex>

class ZRAMDevice
	: public IODevice
{
	using inherited = IODevice;
 public:
	ZRAMDevice();
	~ZRAMDevice() override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

 private:
	// アドレスデコーダ
	uint64 Decoder(uint32 addr) const;

	std::unique_ptr<uint8[]> ram {};

	// アクセス競合を防ぐロック。
	// Lance は裏スレッドからアクセスにくるので。
	std::mutex mtx {};
};

extern std::unique_ptr<ZRAMDevice> gZRAM;
