//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "wxsubwindow.h"
#include "wxtextscreen.h"

class WXLogMonitor : public WXVResizeSubWindow, public IMonitor
{
	using inherited = WXVResizeSubWindow;

 public:
	WXLogMonitor(wxWindow *, wxWindowID);
	virtual ~WXLogMonitor() override;

	nnSize GetMonitorSize() override;
	void MonitorUpdate(TextScreen& ts) override;

 private:
	// サイズ変更イベント
	void OnSize(wxSizeEvent&);

	// マウスホイールイベント
	void OnMouseWheel(wxMouseEvent&);

	// スクロールイベント
	void OnScroll(wxScrollEvent&);

	// スクロールバー再設定
	void SetScroll();

	// スクロール処理
	void DoScroll(int pos);

	// ウィンドウサイズ再計算
	void DoSize() override;

	// バックログに追加
	void Append(const char *);

	// 1行追加された時の下請け処理
	void IncLines();

	// スクロールバー
	wxScrollBar *vscroll = NULL;

	int col = 0;
	int row = 0;

	// 表示開始位置
	// パネル最下行に表示する行を、最新行からのオフセットで表す。
	// 0 ならパネル最下行は最新行、1 なら一つ手前の行…となる。
	int vpos = 0;

	// 現在のログ行数
	int lines = 0;

	// ログ全体の行数(上限)
	const int backlog = 10000;

	// ログバッファ
	// ここは属性フィールドを持たないテキストによる循環バッファ
	std::vector<uint8> logbuf {};

	// 今から書き込む行の先頭 (logbuf からのオフセット)
	int cursor {};

	// UTF-8 -> CP932 変換
	wxCSConv conv { "CP932" };

	// イベントテーブル
	wxDECLARE_EVENT_TABLE();
};
