//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "wxheader.h"
#include <atomic>

class WXMainView
	: public wxPanel
{
	using inherited = wxPanel;
 public:
	WXMainView(wxWindow *);
	virtual ~WXMainView();

	void DoRefresh();
	void DoResize(double scale);

	// 現在のスケールを返す
	double GetScale() const;
	// 現在のスケール (wxapp が初期値を指定するため static 変数にしてある)
	static double screen_scale;

	// マウスモードを設定
	void SetMouseMode(bool enable);
	// マウスモードを取得 (キャプチャされているなら true)
	bool GetMouseMode() const { return mousemode; }
	// アクティベートイベント
	void OnActivate(wxActivateEvent& event);

 private:
	void OnTimer(wxTimerEvent& event);
	void OnPaint(wxPaintEvent& event);
	void OnKeyDown(wxKeyEvent& event);
	void OnKeyUp(wxKeyEvent& event);
	void OnMouse(wxMouseEvent& event);

	// 裏バッファ。
	// VM がレンダリングするバッファ。
	std::unique_ptr<uint8[]> ImageBuf {};

	// 仮想画面サイズ (ピクセル)
	int rendwidth = 0;
	int rendheight = 0;

	// 実画面サイズ (ピクセル)
	int viewwidth = 0;
	int viewheight = 0;

	// 共通キーコードを取得する
	int GetKeyCode(wxKeyEvent& event);

#if defined(__WXGTK__)
	static const int keycode_table[0xd4];
#elif defined(__WXOSX__)
	static const int keycode_table[0x80];
#endif

	// マウスモード (キャプチャしていれば true)
	bool mousemode = false;
	// 前回のマウス座標とボタン
	wxPoint prevpos {};
	bool prevlb {};
	bool prevmb {};
	bool prevrb {};

	// WarpPointer() による移動検出用
	bool motion_by_warp_pointer = false;
	// マウスカーソルを移動させる
	void MyWarpPointer(wxSize pos);

	// 再描画を律速する
	wxTimer timer {};
	std::atomic<bool> paint_request {};

	wxDECLARE_EVENT_TABLE();
};

extern WXMainView *gMainView;
